/*

AD9833 DDS Driver for DC Receiver

OH2NLT, Juha Niinikoski 14.01.2005

Phase reg definitions corrected 19.01.2005
Filter select flip-flop drive added 03.07.2005
AUX output added 06.10.2005
Flip-flop control corrected 13.10.2005

*/

// AD9833 definitions
#define REG_SELECT	(3 << 14)	// Register select mask
#define	FREQ0		(1 << 14)	// Select FREQ0 register
#define	FREQ1		(2 << 14)	// Select FREQ1 register
#define	PHASE0		0xC000		// Select PHASE0 register
#define	PHASE1		0xE000		// Select PHASE1 register

// Control register bits
#define	B28			(1 << 13)
#define	HLB			(1 << 12)
#define	FSELECT		(1 << 11)
#define	PSELECT		(1 << 10)
#define	ADRESET		(1 << 8)
#define	SLEEP1		(1 << 7)
#define	SLEEP12		(1 << 6)
#define	OPBITEN		(1 << 5)
#define	DIV2		(1 << 3)
#define	ADMODE		(1 << 1)

// Mode select combinations
#define AD_RUN		0x0000		// Control register bits for normal Run mode

// Send 16-bits to AD9833 chip
void send_ad9833(unsigned int data)
	{
	char i;

	AD_FSYNC = 1;				// Set start condition
	AD_SCLK = 1;
	AD_FSYNC = 0;				// Start frame

	for (i = 16; i > 0; i--)	// Decrementing loop uses 2 fewer bytes
		{
		if(data & 0x8000)
			AD_SDATA = 1;
		else
			AD_SDATA = 0;

		AD_SCLK = 0;			// Generate clock pulse
		asm("nop");
		AD_SCLK = 1;
		data <<= 1;				// Next bit
		}

	AD_FSYNC = 1;				// End frame
	}

// Send tuning word to AD9833 chip, and display frequency
void tune_ad9833(unsigned long f, char dds_enable)
	{
	unsigned long divisor, scale, dds, quot, num;
	unsigned int temp;
	char decade, dp, i, j = 0;

// The elegant way of implementing the decimal point is:
// dp = (out_of_band_alarm && (blink_timer > BLINK_TIME)) ? ' ' : (vfo_select) ? 0xA5 : '.';
// but this uses 24 bytes more code! A.Ryan 9/MAY/13

	if(out_of_band_alarm && (blink_timer > BLINK_TIME))
		dp = ' ';
	else if(vfo_select)
		dp = 0xA5;
	else
		dp = '.';

	divisor = 1000000UL;				// Start with MHz decade
	scale = 1UL;
	dds = 0;

	for(i = 0; i < 6; i++, divisor /= 10UL, scale *= 10UL)
		{
		decade = (char)(f / divisor);	// Get the next decade value,
		f -= divisor * decade;			// Remove present decade value from frequency

		if(dds_enable)					// This is a frequency change, so,
			{
			num = dds_word[decade];		// get next decade's DDS word
			quot = num / scale;			// and round it.
			num -= quot * scale;		// Calculate the remainder,

			if(num >= (scale >> 1))		// and round the value for optimum frequency accuracy,
				quot++;

			dds += quot;				// and add the present decade to previous decade's DDS word.
			}
		else							// This is a frequency display.
			{
			if((j == 1) || (j == 5))	// If the array index position is 1 or 5, insert a decimal point or blank,
				disp_buffer[j++] = dp;	// Insert in buffer and increment the array index pointer.

			disp_buffer[j++] = decade + '0';// Insert next frequency digit and increment array index pointer.
			}
		}

	disp_buffer[8] = 0;					// Terminate the display buffer

	if(dds_enable)						// If enabled, send data to DDS chip
		{
		temp = (dds >> 14) & 0x3FFF;	// Filter upper 14 bits
		BAND = (temp > FILTER_SEL);		// Next DDS FSYNC rising edge sets the flip-flop
		AUX_OUT = aux_out & (blink_timer >= BLINK_TIME);	// Set AUX output bit (Out-Of-Band/Voltage Alarm Indicator)
		send_ad9833(AD_RUN | HLB);		// Select MSB
		send_ad9833(temp | FREQ0);		// Write to FEQ0 register
		temp = dds & 0x3FFF;			// Filter lower 14 bits
		send_ad9833(AD_RUN);			// Select LSB
		send_ad9833(temp | FREQ0);		// Write to FEQ0 register
		}
	}

// Setup & start AD9833 chip
void Init_ad9833(void)
	{
	send_ad9833(ADRESET);				// Keep chip in reset state	
	send_ad9833(PHASE0);				// Clear PHASE0 register
	tune_ad9833(0, 1);					// Set start up frequency & start DDS
	}

