/*
 Simple LCD driver. 8 bit interface, no R/W signal in use.

 OH2NLT / Juha Niinikoski 14.08.2002
 Modified for DC Receiver display & DDS board 11.01.2005

 S-meter logic (corrected) 12.01.2005
 New code size optimized decimal number conversion. Code size reduced 450 words. 19.01.2005
 HEX nibble display added, code size optimized 04.02.2005
 s-meter code size optimized 07.02.2005
 small code size addjustment 13.02.2005
 selected VFO indicator added 06.10.2005
 rit display added 20.10.2005
*/

// 1x16 TM161ABA6 LCD display memory layout
// Organization is actually 2x8 characters in one display line
#define LINE_1		0x00		// Line 1 start address, first 8 characters
#define LINE_2		0x40		// Line 2 start address, next 8 characters

#define CURSET		0x80		// Set memory cursor
#define ADRSET		0x40		// Set CGRAM address

#define FIRST_HALF	(CURSET + LINE_1)	// First half of display
#define SECOND_HALF	(CURSET + LINE_2)	// Second half of display

#define BUS_IDLE	0xFF		// LCD Data Bus idle state
#define CLR_LCD		0x01

// S-meter constants
#define DISP_LEN	7			// Display length (char) (New Scale)
#define FONT_W		6			// Font width, corrected 12.01.2005
#define MAX_GRAPH	42			// Graph length in pixels for new shorter S-Meter scale
#define FULL_BLOCK	0x06
#define EMPTY_BLOCK	0x00
#define SCALE_END	0x07

unsigned char disp_buffer[9];	// Display buffer used for most displays

void send_to_LCD(void)
	{
	LCD_E = 1;
	asm("nop");
	asm("nop");
	LCD_E = 0;
	}

// Print one character to LCD panel, RS = 1
void lcdputch(unsigned char output)
	{
	LCD_DATA = output;
	LCD_RS = 1;
	send_to_LCD();
	LCD_DATA = BUS_IDLE;		// Leave bus idle state
	delayu(50); 				// About 40 us delay
	}

// Send command to to LCD panel, RS = 0
void lcdcom(char command)
	{
	LCD_DATA = command;
	LCD_RS = 0;
	send_to_LCD();
	LCD_RS = 1;					// Leave RS high
	LCD_DATA = BUS_IDLE;		// Leave bus idle state
	delayu(50); 				// About 40 us delay
	}

#if 0
// Set LCD cursor poition, eg: lcdpos(LINE_1), lcdpos(LINE_2 + 5)
void lcdpos(char position)		// Position cursor
	{
	lcdcom(CURSET + position);
	}
#endif

void lcdclr(void)
// Clear LCD panel & set cursor to 0.0 position
	{
	lcdcom(CLR_LCD);			// Clear command
	delaym(2);					// Long delay 2mS. Manufacturer's spec: 1.64mS min
	}

void clklcd(char output)
	{
	LCD_DATA = output;
	send_to_LCD();
	}

// Initialise the LCD system
void lcdinit(void)
	{
	char i;

	delaym(20);					// 20mS - Manufacturer's spec: 15mS min

	for (i = 3; i > 0; i--)		// Counting down uses 2 less words
		{
		clklcd(0x38);			// LCD reset, 8-bit interface, 1-line, 5*7 font
		delaym(6);				// 6mS - Manufacturer's spec: 4.1mS min
		}

	clklcd(0x08);				// Display Off
	delaym(5);					// 5mS
	clklcd(0x01);				// Display clear
	delaym(5);					// 5mS
	clklcd(0x06);				// Entry mode set
	delaym(5);					// 5mS
	clklcd(0x0C);				// Display On
	delaym(5);					// 5mS
	}

// TM161ABA6 character generator routines
void set_cgram(unsigned char adr, unsigned char dta)	// Set CGRAM address & data
	{	
	adr &= 0x3F;
	lcdcom(adr | ADRSET);		// Set CGRAM address command + address
	lcdputch(dta);
	lcdclr();					// Back to normal mode (clear LCD)	
	}

// Set CHGEN bits. Data, number of sets
void set_ch_bits(char data, char count)
	{
	while(count)
		{
		lcdputch(data);
		count--;				// Auto-decrementing the variable in the while() uses 1 more word.
		}
	}

// Row count must be 8 for each character
/*
 By carefully arranging the special characters for the partial blocks so that they occupy the address
 equal to the remainder, you can then remove two arrays from the original version.
*/
void set_chgen(void)			// Write bar symbols to HD44780 RAM chgen
	{		
	lcdcom(0x00 | ADRSET);		// Set CGRAM address command + address

// CGRAM 00 - Empty block
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x10, 1);	// @....
	set_ch_bits(0x15, 2);	// @.@.@
							// @.@.@
	set_ch_bits(0x10, 1);	// @....
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 01 - Remainder = 1
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x10, 1);	// @....
	set_ch_bits(0x15, 2);	// @.@.@
							// @.@.@
	set_ch_bits(0x10, 1);	// @....
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 02 - Remainder = 2
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x18, 1);	// @@...
	set_ch_bits(0x1D, 2);	// @@@.@
							// @@@.@
	set_ch_bits(0x18, 1);	// @@...
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 03 - Remainder = 3
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x1C, 1);	// @@@..
	set_ch_bits(0x1D, 2);	// @@@.@
							// @@@.@
	set_ch_bits(0x1C, 1);	// @@@..
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 04	- Remainder = 4
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x1E, 1);	// @@@@.
	set_ch_bits(0x1F, 2);	// @@@@@
							// @@@@@
	set_ch_bits(0x1E, 1);	// @@@@.
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 05 - Remainder = 5
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x1F, 4);	// @@@@@
							// @@@@@
							// @@@@@
							// @@@@@
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 06 - Remainder = 6
	set_ch_bits(0x00, 2);	// .....
							// .....
	set_ch_bits(0x1F, 4);	// @@@@@
							// @@@@@
							// @@@@@
							// @@@@@
	set_ch_bits(0x00, 2);	// .....
							// .....

// CGRAM 07 - End-Of-Scale Bar
	set_ch_bits(0x00, 0x02);	// .....
								// .....
	set_ch_bits(0x10, 0x04);	// @....
								// @....
								// @....
								// @....
	set_ch_bits(0x00, 0x02);	// .....
								// .....

	lcdclr();					// Back to normal mode (clear LCD)
	}
/*
 Draw S-meter, scale = 0...42
 The new S-meter scale is 1 character shorter than the original. This is to ensure that
 the scale does not encroach upon the frequency display, which in the previous firmware
 made it look a bit ugly. In order not to lose signal strength data, since the new scale is
 7/8 of the original, the data is re-scaled to fit.
*/
void draw_s_meter(int len)
	{
	char i, whole;

	len *= 7;
	len >>= 3;					// Scale for new S-meter graph length

	if(len > MAX_GRAPH)			// Cut to max
		len = MAX_GRAPH;

	whole = len / FONT_W;		// Number of characters with full blocks
	len -= (whole * FONT_W);	// Partial block
	i = 0;

	do	{
		if(i < whole)			// First display the full blocks,
			lcdputch(FULL_BLOCK);
		else if(len)			// then display any partial block.
			{
			lcdputch(len);
			len = 0;			// Reset len so that next time through it is ignored,
			}
			else
				lcdputch(EMPTY_BLOCK);	// then display the rest of the scale,

		i++;
		} while (i < DISP_LEN);

	lcdputch(SCALE_END);		// and finally display End-Of-Scale bar.
	}

// Put string
void putst(register const char *str)
	{
	while(*str)
		lcdputch(*str++);
	}

// Convert binary number to ASCII string
/*
  bin2ascii() was originally designed to convert long integers to their string representation and inserted
  the ASCII equivalents into the display buffer for subsequent display by the LCD routines. With the
  modification of the firmware to use a decimal frequency system, bin2ascii() was no longer required to
  deal with long integers, only normal integers. The long display was only utilised for the frequency
  display function, and this is accomplished in the frequncy_display() function in ad9833.c

  Since only integers are now converted it made sense to optimise the whole software package to remove
  unneccessary duplication bearing in mind the very small amount of memory available. To this end,
  the function int_display() and bin2ascii() were essentially combined, and int_display(), which was
  only ever used to display either a single character or an unformatted integer, was removed. Ordinarily
  the library routine sprintf() would be called upon to format numbers, but there is insufficient memory
  to accomodate this approach, and so bin2ascii() had to perform all the necessary formatting required.

  The design is based upon the requirement that the only numbers to be formatted will fall into the
  following categories:

  Battery voltage display
  This is an unsigned integer number consisting of a value between a minimum of about 10V and a maximum
  of about 20V. Below 10V the transceiver will not function, and above 15V it is likely to suffer
  damage. The number is obtained from the 10-bit A-D convertor which has a maximum output number of
  1024. The resistors forming the potential divider are chosen such that the full-scale voltage of the
  voltmeter section is 20.48V, corresponding to an A-D output of 1024, giving an incremental resolution
  of 20mV. It is thus easy to see that the actual scale factor is divide by 50. In order to allow for
  resistor and A-D convertor reference voltage tolerances, a calibration constant is chosen to allow
  calibration adjustments in approximately 10mV increments, representing about 1/2LSB compensation.
  
  The actual equation employed is:

  Battery Voltage Display Number = ((A-D * 20480) / Calibration Constant) / 10
  
  For a nominal 13.80V input and a default calibration constant of 1024, this gives:
  
  Battery Voltage Display Number = ((690 * 20480) / 1024) / 10 = 1380
  
  bin2ascii() must therefore display this as: 13.80V
  
  RIT Offset Frequency
  The receiver incremental tune offset frequency is a signed integer having a range of values between
  -9990 and +9990 - well inside the range of an integer.
  
  The number is displayed to 10Hz resolution in the format: RIT+9.99 RIT 0.00 or RIT-9.99
  
  The buffer contents for the various displays are:
    num   0  1  2  3  4  5  6  7  8
	1380     1  3  .  8  0  V     \0	Battery Voltage.			Flag = 3
	1024     1  0  2  4           \0	Battery Calibration Factor.	Flag = 0
	-9990 R  I  T  -  9  .  9  9  \0	Negative RIT Frequency.		Flag = 5
     0000 R  I  T     0  .  0  0  \0	Zero RIT Frequency.			Flag = 5
    +9990 R  I  T  +  9  .  9  9  \0	Positive RIT Frequency.		Flag = 5
  
  The disp_buffer is 9 bytes in length, and the LCD displays 16 characters in two 8 character slices.
  The function is passed the numerical value as signed integer, and a flag. The flag is used to indicate
  the type of display required, as well as the position in the buffer for the decimal point. A flag of 0
  indicates a straight numerical display, and is used with the battery calibration integer during the
  system setup.
*/
void bin2ascii(int num, char flag)
	{
	int decade;
	char c, i = 1;

	if(flag == 5)						// Special RIT display
		{
		disp_buffer[0] = 'R';
		disp_buffer[i++] = 'I';
		disp_buffer[i++] = 'T';

// The elegant way of doing this is: c = (num > 0) ? '+' : (num < 0) ? '-' : ' ';
// if(c == '-') num = -num;
// but this takes more code. A.Ryan 13/MAY/13
		if(num > 0)
			c = '+';
		else if(num < 0)
			{
			num = -num;
			c = '-';
			}
			else
				c = ' ';

		disp_buffer[i++] = c;
		}
	else
		disp_buffer[0] = ' ';			// Leading space for all other displays

	for (decade = 1000; decade > 0; decade /= 10)
		{
		if(i == flag)
			disp_buffer[i++] = '.';		// Insert decimal point

		c = num / decade;				// Calculate next decade digit value
		num -= decade * c;				// Calculate next remainder
		disp_buffer[i++] = c + '0';		// Insert ASCII digits
		}
	
	if(flag == 3)						// Battery Voltage
		disp_buffer[i++] = 'V';

	while(i < 8)
		disp_buffer[i++] = ' ';			// Fill buffer with spaces

	disp_buffer[8] = 0;					// String terminator
	}

