/*
 DDS / Receiver controller for JUMA RX1 Direct conversion Receiver. 
 OH2NLT Juha Niinikoski 12.01.2005

 Processor is Microchip 16F819 providing 2k words Flash, 256 bytes RAM and 256 bytes EEPROM.
 Project compiled under MPLAB7.00 with Hitec PICC 8.05PL2 c-compiler

 This code can be used freely for private Ham Radio projects.
 Limited amount of 16F819 Flash memory has forced the use of code size-effective programming methods.

 DDS driver added, initial display logic 14.01.2005

 Code cleaning & size optimization 18.01.2005 99% Flash used v0.01
 New compacted display conversion routines & EEPROM frequency storage 19.01.2005 78%Flash used v0.02
 Freq set at init, tune digit select corrected and Pulse Encoder logic. 20.01.2005 v0.03
 Appearance corrections, init sequence corrections, 500Hz tuning step, disp rounding, morse test 
 86,1% Flash space used 26.01.2005 v0.04
 "RF" play with 20MHz sub harmonics morse message 27.01.2005 v0.05
 Encoder debounce filter, step change beep logic, RA0 audio out. 89,1% Flash used 03.02.2005 v0.06
 Beep added when entering tune step change mode, code size optimization 04.02.2005 v0.07
 Code size optimization, tune step save to EEPROM, long push save logic. 90,5%  07.02.2005 v0.08
 Saved text added to save logic 91,1% 10.02.2005 v0.09
 Code size & other adjustments, different high limit for test mode 92,5% 13.02.2005 v0.10

 Start up message display time changed, first release version 24.02.2005 v1.00
 700 Hz CW TX-shift  logic added 20.06.2005 v2.00
 Filter select flip-flop drive added 03.07.2005 v3.00 / JUMA TRX-1 PCB reguired for this function
 Encoder clear after startup, 100Hz step 15.08.2005 v3.01

 Project changed to PIC16F88 processor 06.10.2005 v3.02
 VFO A/B  select added 06.10.2005 v3.02
 RX / TRX select added 07.10.2005 v3.03
 Band filter select flip-flop action corrected 13.10.2005 v3.04

 RIT test 20.10.2005 v3.04r

 TRX1 code release patches 10.05.2006, Version number changed to TRX1 v1.00r
 69,3% Flash & 29,3% RAM used

- RIT operating mode minor display text changes
- S-meter overflow inhibit
- Band tune limit logic changed, tuning stops at limits
- Operating mode select logic added, CV length & VFO & RIT select logic added
  Start the test mode and adjust operating mode while holding encoder switch pressed.
  Bit 0 = RIT, Bit 1 = Short CV, Bit 2 = Long CV, Bit 1 & 2 = Beep tuning step
- Do not change step while RIT on
================================================================================================
Modifications by Adrian Ryan, 5B4AIY March 2013
================================================================================================
 AD9833
 If the output frequency is Fout, the reference frequency Fref, and the command word DDS, then:

 DDS = (Fout * 2^28) / Fref

 Fout = DDS * Fref / 2^28

 If Fref = 20,000,000Hz
 For an output frequency of 1,000,000Hz,

 DDS  = 1,000,000 * 2^28 / 20,000,000
      = 1,000,000 * 268,435,456 / 20,000,000
	  = 13,421,772
	  = 0xCC,CC,CC

 Fout = 13,421,772 * 20,000,000 / 268,435,456
      = 999,999.940,395,355,224,609,375Hz
 An error of: 0.059,604,644,775,390,625Hz, or -6 parts in 10^8.

 For a DDS register of 1, the output frequency is:
 Fout = 1 * 20,000,000 / 2^28
      = 0.074,505,805,969,238,281,25Hz

 The original firmware calculated the DDS word in DDS units, adding and subtracting the frequency increment
 as the VFO knob is rotated. In order to resolve the frequency to decimal for the display, the DDS word was
 multiplied by a constant, and the upper 4 bytes of the resultant 64-bit word used as the frequency.

 As an example, assume that the desired output frequency of the DDS is 5.000,000MHz. This corresponds to a
 DDS word of: 5,000,000 * 2^28 / 20,000,000 = 67,108,864 or 0x04,00,00,00.
 
 The constant used is: 0.074,505,805,969,238,281,25 * 2^32 = 320,000,000.
 
 By multiplying the DDS word by this constant we get the 64-bit (8-byte) result: 67,108,864 * 320,000,000
 = 21,474,836,480,000,000 or 0x00,4C,4B,40,00,00,00,00
 Only the top four bytes are used, the remainder are discarded, thus the binary value is: 0x00,4C,4B,40 or 5,000,000.
 Since using only the top 4 bytes also corresponds to dividing by 2^32, the same result can be calculated
 as: 21,474,836,480,000,000 / 4,294,967,296 = 5,000,000
 
 As the encoder frequency increment and decrement calculations are carried out using the DDS word, the
 frequency increments are only approximately representable in DDS units, and thus  there will be an
 accumulating approximation error with each increment/decrement. For example, the 10Hz minimum frequency
 increment is represented by the value: 134. This corresponds to an actual frequency increment of:
 9.983,777,999,877,929,6875Hz.

 After 616 increments, which should be 6,160Hz, the actual increment will have been:
 6,150.007,247,924,804,687,5Hz, an error of: -9.992,752,075,195,312,5Hz.

 Similarly, the 100Hz increment is represented by: 1,342. This corresponds to an actual increment
 of: 99.986,791,610,717,773,437,5Hz. After 7,570 increments, which should be 75,700Hz, the actual increment
 would have been: 756,900.012,493,133,544,921,875Hz, an error of: 99.987,506,866,455,078,125Hz.
 The 1kHz increment is represented by: 13,421. This is an actual frequency increment of: 999.942,421,913,146,972,656,25Hz.
 An error of -0.057,578,086,853,027,343,75Hz. Since the total frequency range of the synthesiser is from 0 - 8MHz,
 this error will not be accumulated sufficiently to be displayed.

 In fact, the user is not concerned that the frequency changes exactly correspond to encoder steps, and thus
 additional step to correct the frequency will not be noticed. However, intermediate frequency steps will
 occaison some error, and an analysis using all the steps from 10Hz to 8.0MHz in 10Hz increments shows
 that there will be errors of +/-5Hz at some frequencies. Even so, this error is well within the limits
 expected for commercial amateur radio equipment. Nonetheless, the synthesiser is capable of generating
 frequencies with a lower error than this. Ideally, we should employ 32-bit arithmetic with a 64-bit product
 but in an 8-bit machine this would entail a considerable amount of logic to implement. Fortunately there
 is a simpler alternative - a look-up table. In its simplest implementation, we would store every possible
 frequency from 10Hz - 8.000000MHz in an array, and use the frequency as an index to retrieve the corresponding
 32-bit DDS word. Since each 32-bit word occupies 4 bytes, and we would need 800,000, this give an array size
 of 3.2Mbytes! If this were a Windows environment, I can well imagine a sloppy programmer actually doing this!
 However, if we split the table up into its decades, then the MHz decade would require 9 values from 0 - 8MHz,
 the 100kHz decade 10 values from 0 - 900kHz, and so on down to the 10Hz decade. By extracting each decade's
 value from the frequency and using this as an index into the respective tables, we simply sum the extracted
 32-bit words to form the final DDS word. This would reduce the array size to 236 bytes. Indeed, with a little
 more programming effort, even the zero entry could be removed thus reducing the table size to 204 bytes.
 An examination of the various values in the tables reveals that even this is unneccessary. The MHZ table actually
 contains all the values required. For example, take the 5MHz digit:
 
	===================
	Frequency	DDS Word
	===================
	5MHz		67108865
	500kHz		6710886
	50kHz		671088
	5kHz		67108
	500Hz		6710
	50Hz		671
	===================
 
 Consequently, by extracting the DDS word from the MHz decade and dividing it by the appropriate factor, we can
 accumulate the same DDS word, but with far less data space occupied. In fact, the total array size now shrinks
 to a mere 40 bytes. Indeed, even the zero element could be eliminated bringing it down to 36 bytes. Whether
 it is worth this refinement is debatable, since there would have to be additional lines of program code to skip
 a zero decade value. By introducing an integer rounding routine, the accuracy of this method can be improved, and
 an analysis was performed and it was found that examining every frequency from 10Hz - 8MHz in 10Hz increments,
 the maximum error was +0.173Hz and -0.218Hz, an improvement of 20 times.
 
 If rounding is not used, then for a frequency of 5.555,550Mhz, the exact DDS word is: 74,565,329.879,04
 This would be rounded to an integer to give: 74,565,330, and give an output frequency of: 5,555,550.009,012Hz
 If the non-rounded value were used, i.e, it was truncated to: 74,565,329, this would give an output frequency
 of: 5,555,549.934,506Hz
 
 Using the above table entries: 67,108,865 + 6,710,886 + 671,088 + 67,108 + 6,710 + 671 = 74,565,328
 This would give an output frequency of: 5,555,549.860,00Hz, corresponding to an error of -0.14Hz.
 If rounding were used, then the DDS word would be: 67,108,865 + 6,710,887 + 671,089 + 67,109 + 6,711 + 671 = 74,565,332
 giving an output frequency of: 5,555,550.158,024, an error of +0.158Hz
 
 If we compare these frequencies with the actual output frequency of: 5,555,549.934,506Hz, then without rounding
 the output frequency of: 5,555,549.860,00Hz, an error of: -0.75Hz, with rounding, the error is: 0.224Hz. In fact,
 the actual errors overall are less than this for the vast majority of frequencies generated.

 Button Logic
 In order to implement these additional features, the button logic has to be modified. To achieve this,
 in the 1mS main interrupt there is a variable button_count, as well as a mode indicator _mode. In the
 starting state, the operating mode is 0, and the exit mode is 0. If the button is pressed and held,
 then each time the 1mS interrupt occurs the button_count will increment. When the button is released
 the count is compared with the SHORT/MEDIUM/LONG count values, and if the count is greater than each
 of these then the _mode indicator is incremented. If this is the initial state then operating mode is 0
 and thus it will be set to the same value as _mode. The exit state is still at 0. In the main loop,
 the operating mode is examined, and by means of a switch/case construct, the desired operating mode
 is selected.
 
 If the button is now pressed again, this will cause the exit state indicator to be set to an exit value.
 The code in the DO-WHILE loop will examine the exit mode and determine the next actions. If it is exit mode 1
 this corresponds to a normal return to Mode 0. The exit will occur, and the operating mode and exit mode
 reset to zero.
 
 If the exit mode is 2, and the operaating mode is Mode 1, then this corresponds to the copy VFO function.
 The source VFO is determined by the current vfo_select. If vfo_select is 0, the ACTIVE VFO is A, and thus
 the contents of VFO-B will be copied to VFO-A. Similarly if the ACTIVE VFO is B, then the contents of VFO-A
 will be copied. This method was chosen so that the active VFO's frequency display will change to show you
 that something has happened. There is also a different beep tone depending upon which copy operation has
 taken place.

 If the exit mode is 3, then the current transceiver state will be written to the EEPROM.

 Thus various operating modes will be selected as desired. The default state is Mode 0, in this state
 the transceiver will display the S-Meter and frequency in receive, or power meter and frequency in transmit.
 In this state it will respond to the PTT, CW/DSB, RIT switches as appropriate.

 In Mode 1, the user can select either VFO-A or VFO-B by rotating the VFO knob. The preset frequency for the
 selected VFO will be displayed. A brief press of the VFO button will restore Mode 0 with the new VFO selected.

 In Mode 2, the display will show the current VFO increment, 10/100/1kHz/5kHz and it can be altered by rotating
 the VFO knob to select the desired increment. Pressing the VFO knob will then exit this mode and return
 to Mode 0. The 5kHz increment was chosen so that when the receiver is used in its general coverage mode, this
 corresponds to the standard channel spacing of commercial SW broadcasting stations.

 In Mode 3, the rapid bandswitch feature is engaged. The display will show the current band, 160/80/40
 and the actual frequency, and rotating the VFO knob will select the new band. A press of the VFO buutton will
 effect the change, and return you to Mode 0.

 In all exit modes, a long push will save the current transceiver's state, including the currently selected VFO,
 the frequencies of VFO A & B, and the current step rate.

Version 5.33
Changed logic so that RIT switch now is hard-wired to select RIT On/Off, and
VFO-A/VFO-B is now always selected via the VFO button push logic. As a result,
the opr_mode variable now only deals with IARU/USA band selection and beep
On/Off. Added logic to take care of out-of-band indication when in CW mode:
Memory Summary:
    Program space        used   F58h (  3928) of  1000h words   ( 95.9%)
    Data space           used    95h (   149) of   170h bytes   ( 40.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added full prompts for System Setup:
Memory Summary:
    Program space        used   F8Fh (  3991) of  1000h words   ( 97.4%)
    Data space           used    9Ah (   154) of   170h bytes   ( 41.8%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Optimised tuning_increment[] array, and used lcdclr() instead of positioning command:
Memory Summary:
    Program space        used   F8Bh (  3987) of  1000h words   ( 97.3%)
    Data space           used    9Ah (   154) of   170h bytes   ( 41.8%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Removed superflous dds_freq assignment, changed the order of the modes. Mode 1 is now
VFO A/B Select, Mode 2 is now Battery Voltage & Tune Increment Select:
Memory Summary:
    Program space        used   F7Bh (  3971) of  1000h words   ( 97.0%)
    Data space           used    9Ah (   154) of   170h bytes   ( 41.8%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added 5kHz tuning step. 5kHz step chosen so that if the receiver is used as a general
coverage SW receiver, this corresponds to the standard channel spacing. Changed loop
counter in ad9833.c display_freq() function to use the display buffer index rather
than the divisor:
Memory Summary:
    Program space        used   F9Bh (  3995) of  1000h words   ( 97.5%)
    Data space           used    9Bh (   155) of   170h bytes   ( 42.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Combined case 8/9 code sections in System Setup:
Memory Summary:
    Program space        used   F97h (  3991) of  1000h words   ( 97.4%)
    Data space           used    9Bh (   155) of   170h bytes   ( 42.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Combined case 4/5/6 code sections in System Setup:
Memory Summary:
    Program space        used   F17h (  3863) of  1000h words   ( 94.3%)
    Data space           used    9Bh (   155) of   170h bytes   ( 42.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.0
Unfortunately, the decimal calculation system did not work. The compiler simply
cannot deal with 64-bit products. Stripped out all associated code, including the
XTAL frequency compensations code as well as various other sections, re-organised the
VFO memories etc:
Memory Summary:
    Program space        used   E60h (  3680) of  1000h words   ( 89.8%)
    Data space           used    9Dh (   157) of   170h bytes   ( 42.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Optimised System Setup:
Memory Summary:
    Program space        used   E56h (  3670) of  1000h words   ( 89.6%)
    Data space           used    9Dh (   157) of   170h bytes   ( 42.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Optimised Main Loop, Mode 0, Mode 1, and Mode 2:
Memory Summary:
    Program space        used   E4Bh (  3659) of  1000h words   ( 89.3%)
    Data space           used    9Dh (   157) of   170h bytes   ( 42.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added dds_word[] array, added basic frequency set code:
Memory Summary:
    Program space        used   EEDh (  3821) of  1000h words   ( 93.3%)
    Data space           used    AEh (   174) of   170h bytes   ( 47.3%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added DDS word calculation code:
Memory Summary:
    Program space        used   F3Ch (  3900) of  1000h words   ( 95.2%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Fixed beep tone bug - time was 10x times too long, as a result of a left-over code from a
a previous modification that was subsequently removed:
Memory Summary:
    Program space        used   F30h (  3888) of  1000h words   ( 94.9%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added rounding to DDS word calculation:
Memory Summary:
    Program space        used   F41h (  3905) of  1000h words   ( 95.3%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.01
Added double beep for VFO A-B copy acknowledgement:
Memory Summary:
    Program space        used   F74h (  3956) of  1000h words   ( 96.6%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.02
Fixed transmit/receive mode bug. The flag dsb_mode was not being updated in the interrupt.
Unfortunately adding just 1 line of code caused out-of-memory errors, and these could only be
resolved by removing all the checksum code in the read_freq() and save_freq() EEPROM routines.
This means that the default values have to be manually loaded. Howver, since the default load
values are only ever loaded at the very first startup after flashing the chip, this is not a
real hardship, and after presetting the values the program memory and the EEPROM memory
contents can be saved to a load file:
Memory Summary:
    Program space        used   FA7h (  4007) of  1000h words   ( 97.8%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added initialisation of VFO-A and VFO-B to avoid difficulties when starting from a blank EEPROM:
Memory Summary:
    Program space        used   FB7h (  4023) of  1000h words   ( 98.2%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Modified read_freq() and save_freq() to try and eliminate battery cal reset to zero bug:
Memory Summary:
    Program space        used   FBAh (  4026) of  1000h words   ( 98.3%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Modified read_freq() and save_freq() to eliminate even the use of local variables.
Renamed rit_sign as gp_var1, it is no longer used as the sign indicator for the special
RIT display:
Memory Summary:
    Program space        used   FB8h (  4024) of  1000h words   ( 98.2%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.03
Removed long variables step and temp from main() and used direct references:
Memory Summary:
    Program space        used   F4Fh (  3919) of  1000h words   ( 95.7%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.04
Added vcc_alarm and out_of_band_alarm now OR'ed to aux_out so that the front panel
LED will blink for either an out-of-band or an out-of-range supply voltage:
Memory Summary:
    Program space        used   F56h (  3926) of  1000h words   ( 95.8%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Optimised blink_timer operations in dcrxtim.c:
Memory Summary:
    Program space        used   F53h (  3923) of  1000h words   ( 95.8%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Fixed tone output bug:
Memory Summary:
    Program space        used   F66h (  3942) of  1000h words   ( 96.2%)
    Data space           used    B8h (   184) of   170h bytes   ( 50.0%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.05
The VFO copy beeps shortened, and beep length variable changed from int to
unsigned char, fixed S-Meter bug:
Memory Summary:
    Program space        used   F86h (  3974) of  1000h words   ( 97.0%)
    Data space           used    ABh (   171) of   170h bytes   ( 46.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.06
Re-designed draw_s_meter():
Memory Summary:
    Program space        used   F81h (  3969) of  1000h words   ( 96.9%)
    Data space           used    ABh (   171) of   170h bytes   ( 46.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.07
VFO Copy beeps set to standard 50mS length:
Memory Summary:
    Program space        used   F81h (  3969) of  1000h words   ( 96.9%)
    Data space           used    ABh (   171) of   170h bytes   ( 46.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.08
Added HI/LO V! message:
Memory Summary:
    Program space        used   FE9h (  4073) of  1000h words   ( 99.4%)
    Data space           used    BCh (   188) of   170h bytes   ( 51.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added RIT cancel:
Memory Summary:
    Program space        used   FF1h (  4081) of  1000h words   ( 99.6%)
    Data space           used    BCh (   188) of   170h bytes   ( 51.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Changed offset from a long to an int, removed gp_var1:
Memory Summary:
    Program space        used   FBCh (  4028) of  1000h words   ( 98.3%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.08
Modified S-meter scale to have an end-of-scale bar, and re-scaled S-meter value
to take account of new scale length:
Memory Summary:
    Program space        used   FD7h (  4055) of  1000h words   ( 99.0%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.09
This uses short beeps for the VFO copy, and a scaled S-meter:
Memory Summary:
    Program space        used   FD7h (  4055) of  1000h words   ( 99.0%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.1a	Scaled S-Meter, Short VFO Copy beeps
Version 6.1b	Scaled S-Meter, Standard VFO Copy beeps
Memory Summary:
    Program space        used   FD7h (  4055) of  1000h words   ( 99.0%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.1c	Truncated S-Meter, Short VFO Copy beeps
Version 6.1d	Truncated S-Meter, Standard VFO Copy beeps
Memory Summary:
    Program space        used   FC1h (  4033) of  1000h words   ( 98.5%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.2
Removed redundant set_timeout_u() function in dcrxtim.c:
Version 6.2a	Scaled S-Meter, Short VFO Copy beeps
Version 6.2b	Scaled S-Meter, Standard VFO Copy beeps
Memory Summary:
    Program space        used   FCFh (  4047) of  1000h words   ( 98.8%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.2c	Truncated S-Meter, Short VFO Copy beeps
Version 6.2d	Truncated S-Meter, Standard VFO Copy beeps
Memory Summary:
    Program space        used   FB9h (  4025) of  1000h words   ( 98.3%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.3a
Removed VFO copy beeps, S-Meter is scaled:
Memory Summary:
    Program space        used   F9Ch (  3996) of  1000h words   ( 97.6%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.3b
Removed VFO Copy beeps, S-Meter is truncated:
Memory Summary:
    Program space        used   F86h (  3974) of  1000h words   ( 97.0%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.4a
Added S-Meter peak hold, S-Meter is scaled:
Memory Summary:
    Program space        used   FDAh (  4058) of  1000h words   ( 99.1%)
    Data space           used    BDh (   189) of   170h bytes   ( 51.4%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.4b
Added S-Meter peak hold, S-Meter is truncated:
Memory Summary:
    Program space        used   FC4h (  4036) of  1000h words   ( 98.5%)
    Data space           used    BDh (   189) of   170h bytes   ( 51.4%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.5a
Removed S-Meter peak hold function. It makes tuning an antenna difficult when
checking the reverse power. Fixed band limits bug. The Rapid Band-Switch
frequency limits were always being set to the IARU limits:
Memory Summary:
    Program space        used   FA5h (  4005) of  1000h words   ( 97.8%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Restored Juha & Matti's callsigns during startup:
Memory Summary:
    Program space        used   FD5h (  4053) of  1000h words   ( 99.0%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Removed redundant assignment in region setting code:
Memory Summary:
    Program space        used   FD2h (  4050) of  1000h words   ( 98.9%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Removed redundant assignment in beep on/off setting code:
Memory Summary:
    Program space        used   FD0h (  4048) of  1000h words   ( 98.8%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.5b
Modified the System Setup so that the region setting is before the rapid band-switch
frequency settings, so that a region change now updates the band limit frequencies,
and, to preserve the same array indices, the CW Offset and Battery Calibration settings
were also moved. A redundant lcdclr() removed, and display_freq() optimised:
Memory Summary:
    Program space        used   FCAh (  4042) of  1000h words   ( 98.7%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.5c
Changed Hi/Lo V so that the message: "HI/LO V!" blinks instead of the
voltage display. Added a latched alarm. If the voltage is outside of the
tolerance band, then the alarm is raised and latched. The last measurement
is preserved. The alarm can be reset by briefly pressing the VFO button.
Modified System Setup so that the active VFO, the VFO frequencies, the
current step rate and beep settings are not altered. This allows entry into
the System Setup routine to edit the band switch frequencies and voltmeter
calibration without disturbing the other settings:
Memory Summary:
    Program space        used   FC2h (  4034) of  1000h words   ( 98.5%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Fixed battery calibration factor corruption bug on initial start-up with a
blank EEPROM (tune_digit out of range). Made voltage display continuous when
in the alarm mode:
Memory Summary:
    Program space        used   FC0h (  4032) of  1000h words   ( 98.4%)
    Data space           used    BAh (   186) of   170h bytes   ( 50.5%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added battery voltage averaging over 16 samples:
Memory Summary:
    Program space        used   FEAh (  4074) of  1000h words   ( 99.5%)
    Data space           used    BDh (   189) of   170h bytes   ( 51.4%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Concatenated battery voltage scaling and correction calculation:
Memory Summary:
    Program space        used   FDAh (  4058) of  1000h words   ( 99.1%)
    Data space           used    BDh (   189) of   170h bytes   ( 51.4%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Fixed battery voltage glitch caused by tone output in Mode 2 when step rate
is changed. The A-D input and the tone output use the same pin, thus when
the tone is output the voltage on C14 changes, and it takes time for it
to stabilise to the correct DC input voltage level for the battery voltage
measurement cycle. By inhibiting the measurement for a few hundred mS, this
annoying 'glitch' of the voltage is supressed:
Memory Summary:
    Program space        used   FF1h (  4081) of  1000h words   ( 99.6%)
    Data space           used    BEh (   190) of   170h bytes   ( 51.6%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Restored button count check in dcrxtim.c so that an execssivley long press
of the VFO button merely sets the count to maximum. This code was removed
to make room for the battery voltage averaging feature, but with the savings
in memory obtained by tidying up other code, it can now be replaced:
Memory Summary:
    Program space        used   FDEh (  4062) of  1000h words   ( 99.2%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
	
Version 6.5d
Initial auto-reset vcc_alarm, samples set to 16:
Memory Summary:
    Program space        used   FE3h (  4067) of  1000h words   ( 99.3%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Changed samples to 32 to lessen slight jitter at alrm threshold:
Memory Summary:
    Program space        used   FE3h (  4067) of  1000h words   ( 99.3%)
    Data space           used    C0h (   192) of   170h bytes   ( 52.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added hysteresis to low battery alarm threshold:
Memory Summary:
    Program space        used   FF4h (  4084) of  1000h words   ( 99.7%)
    Data space           used    C2h (   194) of   170h bytes   ( 52.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added step change inhibit when RIT is on, although this meant removing the
button count range check to make room:
Memory Summary:
    Program space        used   FEFh (  4079) of  1000h words   ( 99.6%)
    Data space           used    C2h (   194) of   170h bytes   ( 52.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Moved ad_inhibit to tone() function, moved out_of_band_alarm set to out_of_band()
function:
Memory Summary:
    Program space        used   FE7h (  4071) of  1000h words   ( 99.4%)
    Data space           used    C2h (   194) of   170h bytes   ( 52.7%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Added hysteresis to upper voltage check:
Memory Summary:
    Program space        used   FFCh (  4092) of  1000h words   ( 99.9%)
    Data space           used    C4h (   196) of   170h bytes   ( 53.3%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.5f
Changed the design so that the band number for the Rapid Band-Switch feature is
now calculated in the band_number() function and truncates the frequency to an
integer, thus avoiding two long integer comparisons. Removed redundant assignment
of 0Hz to the dds_freq on start-up, this assignment is already done in the
Init_ad9833() function, restored button count limit check. Moved A-D inhibit
code so that only the voltmeter is inhibited, the S-meter continues to update:
Memory Summary:
    Program space        used   FF0h (  4080) of  1000h words   ( 99.6%)
    Data space           used    C4h (   196) of   170h bytes   ( 53.3%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.6b
Combined display_freq() and tune_ad9833() as they shared a lot of common code,
replaced display_freq() with a paramaterised macro defined in jumatrx1.h:
Memory Summary:
    Program space        used   FC9h (  4041) of  1000h words   ( 98.7%)
    Data space           used    C7h (   199) of   170h bytes   ( 54.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 6.6c
Added code to tune_ad9833() function so that when it displays frequency it skips
the change frequency code and vice versa. Changed FOR/NEXT loop in send_ad9833()
to a decrementing loop, added ds_freq initialisation at start-up, modified code
in bin2ascii() to use more elegant tertiary:
Memory Summary:
    Program space        used   FDDh (  4061) of  1000h words   ( 99.1%)
    Data space           used    C7h (   199) of   170h bytes   ( 54.1%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 7.0c
Modified the out_of_band() function, added alarm enable/disable to setup:
Memory Summary:
    Program space        used   FD0h (  4048) of  1000h words   ( 98.8%)
    Data space           used    C9h (   201) of   170h bytes   ( 54.6%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)

Version 7.1a
Added auto reset enable/disable, and fixed RIT -ve display bug, but had
to remove button_count maximum value test:
Memory Summary:
    Program space        used   FF1h (  4081) of  1000h words   ( 99.6%)
    Data space           used    CBh (   203) of   170h bytes   ( 55.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Removed dsb_mode bit and used direct logic evaluation expression:
Memory Summary:
    Program space        used   FECh (  4076) of  1000h words   ( 99.5%)
    Data space           used    CBh (   203) of   170h bytes   ( 55.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Restored button_count maximum value test:
Memory Summary:
    Program space        used   FF7h (  4087) of  1000h words   ( 99.8%)
    Data space           used    CBh (   203) of   170h bytes   ( 55.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
Removed beep_disable flag, only ever used once, replaced with logic test,
altered low voltage alarm point to 10.70V and high voltage alarm to 15.30V,
fixed intermittent beep when cancelling an RIT offset:
Memory Summary:
    Program space        used   FF1h (  4081) of  1000h words   ( 99.6%)
    Data space           used    CBh (   203) of   170h bytes   ( 55.2%)
    EEPROM space         used     0h (     0) of   100h bytes   (  0.0%)
    Configuration bits   used     1h (     1) of     2h words   ( 50.0%)
    ID Location space    used     2h (     2) of     4h bytes   ( 50.0%)
*/

/*
 EEPROM address definitions
 Address  0 - 3		VFO-A Frequency
 Address  4 - 7		VFO-B Frequency
 Address  8 - 11	160m Frequency
 Address 12 - 15	80m Frequency
 Address 16 - 19	40m Frequency
 Address 20 - 23	CW Shift Frequency
 Address 24 - 25	Battery Calibration
 Address 26			Tune Digit
 Address 27			Operating Mode
*/

//**********************
// Include header files
//**********************
//#include	<pic.h>						// This file is not necessary as the file htc.h has already been included in jumatrx1.h
#include	"jumatrx1.h"

//******************
// Global Variables
//******************
// Constants
const char Settings_Msg[] = {" Setting"};
const char Saved_Msg[] = {"s Saved "};
const char Juma_TRX[] = {"JUMA-TRX"};
const char Version[] = {"-1 v7.1a"};
const char OH2NLT[] = {"  OH2NLT"};
const char OH7SV[] = {"-OH7SV  "};
const char System_Msg[] = {"  System"};
const char Setup_Msg[] = {" Setup  "};
const char Step_Msg[] = {"Step:"};
const char Hi_Lo_V[] = {"HI/LO V!"};
const char Blank_Line[] = {"        "};

const char *VFO_Msg[] = {
						" VFO-A: ",
						" VFO-B: "
						};

const char *Region_Msg[] = {			// Indexed by: temp in System Setup
						" IARU   ",
						" USA    "
						};

const char *On_Off[] = {				// Indexed by: temp in System Setup
						" Off    ",
						" On     "
						};

const char *setup_prompt[] = {			// Indexed by: System Setup, RIT On/Off-VFO A/B Select, Rapid Band-Switch
						" Region:",		// 0 - Region Select, IARU/USA
						"   Beep:",		// 1 - Beep On/Off
						"  Alarm:",		// 2 - Out-Of-Band Alarm Enable/Disable
						"   Auto:",		// 3 - Auto Alarm Reset Enable/Disable
						"   160m:",		// 4 - 160m preset frequency
						"    80m:",		// 5 - 80m preset frequency
						"    40m:",		// 6 - 40m preset frequency
						" Offset:",		// 7 - CW Shift Frequency
						""				// 8 - Dummy, place holder. Battery calibration
						};

const char *tuning_increment[] = {		// Indexed by: tune_digit
						" 10",			// 0
						"100",			// 1
						" 1K",			// 2
						" 5K"			// 3
						};

const int tuning_step[] = {				// Indexed by: tune_digit
						10,				// 0
						100,			// 1
						1000,			// 2
						5000			// 3
						};
// Generate beep tone according tune step 
const int tune_tone[] = {				// Indexed by: tune_digit
						LOW_TONE,		// 0 - 10Hz rate
						MID_TONE,		// 1 - 100Hz rate
						HIGH_TONE,		// 2 - 1kHz rate
						HIGH2_TONE		// 3 - 5kHz rate
						};
/*
 The DDS word is calculated as:
 
 DDS = Frequency * 2^28 / 20,000,000
 The values are rounded to the nearest integer. The values for the 100kHz decade is simply the
 MHz decade divided 10. The 10kHz decade is the MHz decade divided by 100, and so on.
*/
const unsigned long dds_word[] = {		// DDS Words for all decades
						0UL,			// 0 MHz
						13421773UL,		// 1 MHz
						26843546UL,		// 2 MHz
						40265318UL,		// 3 MHz
						53687091UL,		// 4 MHz
						67108864UL,		// 5 MHz
						80530637UL,		// 6 MHz
						93952410UL,		// 7 MHz
						107374182UL,	// 8 MHz
						120795955UL		// 9 MHz
						};
// Variables
unsigned long lower_band_limits[] = {	// Indexed by: band in out_of_band()
						IARU_160M_LOWER,// 0 - 160m (Changed to USA_160M_LOWER for USA use)
						IARU_80M_LOWER,	// 1 - 80m
						IARU_40M_LOWER	// 2 - 40m
						};

unsigned long upper_band_limits[] = {
						IARU_160M_UPPER,// 0 - 160m
						IARU_80M_UPPER,	// 1 - 80m (Changed to USA_80M_UPPER for USA use)
						IARU_40M_UPPER	// 2 - 40m (Changed to USA_40M_UPPER for USA use)
						};

long dds_freq;						// Frequency in use

int rit;							// Changed from long to an int.
int ms_timer;						// Millisecond timer, decremented by IRQ routine, stop at 0
int blink_timer = 0;				// Timer for Out-Of-Band LED blink
int batt_voltage = 1380;			// Presetting this prevents spurious voltage alarm on start-up
int low_batt = LOW_BATT;
int high_batt = HIGH_BATT;

unsigned int batt_avg = 0;

signed char tune_digit;				// Digit to tune
signed char old_tune_digit;
signed char pwctr = 0;				// Encoder delta

char samples = AVG_SAMPLES;			// Battery voltage average samples counter
char opr_mode;						// Operating mode bits

unsigned char ad_inhibit = 0;		// A-D inhibit while voltage settles after tone output
unsigned char portb_copy;			// PORTB read data from IRQ routine. (dcrxtim.c) Update interval = 1mS
unsigned char smeter;				// S-meter data from A/D converter. Smax = 240mV. Also used in System Setup

bit	aux_out;						// Aux output bit. Transferred to AUX output with DDS write - LED alarm indicator.
bit vfo_select;						// VFO A/B select flag. 0 = A, 1 = B
bit rit_select;						// RIT On/Off. 1 = On, 0 = Off
bit vcc_alarm = 0;					// Supply voltage alarm. 0 = No Alarm, 1 = Low/High Voltage
bit out_of_band_alarm = 0;			// Out-Of-Band alarm. 0 = Within Amateur Band, 1 = Out Of Band

union								// VFO data
	{
	long vfos[NUM_VFO];				// VFO-A = 0, VFO-B = 1
	char vfo_bytes[VFO_LEN];		// This structure is used to read and write the frequencies from/to the EEPROM
	} vfo;

/*
 You would think that reducing the size of this union to 1 word, and creating a temporary union in
 measure_all() to save the A-D convertor value would actually reduce the amount of memory required,
 but in fact it increases it by 12 bytes! A.Ryan 30/APR/2013
*/
union								// Battery voltage from A/D converter
	{
	int battery[2];					// Battery Voltage Calibration Constant
	char batregs[4];
	} bat;

//*****************
// Include C files
//*****************

#include	"dcrxtim.c"
#include	"dcrxlcd.c"
#include 	"dcrxadc.c"
#include	"ad9833.c"

//******************
// Processor config
//******************

//	__CONFIG(UNPROTECT & HS & WDTDIS & PWRTEN & BORENON & LVPDIS & MCLRDIS);	// 16F88 config
	__CONFIG(CP_OFF & FOSC_HS & WDTE_OFF & PWRTE_ON & BOREN_ON & LVP_OFF & MCLRE_OFF);
//	__CONFIG(2, FCMDIS & IESODIS);
// CONFIG2 does not work with 805PL2 compiler, patch manually

	__IDLOC(0007);					// Device ID

//******************
// Helper functions
//******************

// Generate tone RA0 pin (Battery voltage measurement pin)
void tone(int pitch, unsigned char len)		// Pitch = 1/2 cycle in uSec units, len = 1mS increments
	{
	if((opr_mode & BEEP_ENABLE))
		{
		ad_inhibit = A_D_INHIBIT_DELAY;
		ms_timer = len;
		TONE_DIR = 0;					// Make tone output active

		while(ms_timer)					// Decremented in tmr2_int() in dcrxtim.c
			{
			TONE_OUT = 1;
			delayu(pitch);
			TONE_OUT = 0;
			delayu(pitch);
			}

		TONE_DIR = 1;					// Return to input mode
		}
	}

// Store & read frequency to/from internal EEPROM
/*
 EEPROM[0 - 3]		= VFO-A
 EEPROM[4 - 7]		= VFO-B
 EEPROM[8 - 11]		= 160m Preset Frequency
 EEPROM[12 - 15]	= 80m Preset Frequency
 EEPROM[16 - 19]	= 40m Preset Frequency
 EEPROM[20 - 23]	= CW Shift frequency
 EEPROM[24 - 25]	= Battery Calibration
 EEPROM[26]			= Tune Digit
 EEPROM[27]			= Operating Mode
*/
void save_freq(void)
	{
	char i;

	for (i = 0; i < VFO_LEN; i++)
		EEPROM_WRITE(i, vfo.vfo_bytes[i]);

	do	{					// DO/WHILE loop is shorter than a FOR/NEXT loop in this instance.
		EEPROM_WRITE(i, bat.batregs[i - BATT_REGS_OFFSET]);	// Save Battery Calibration value
		i++;
		} while(i < TUNE_DIGIT);

	if(vfo_select)
		opr_mode |= VFOB_SELECT;			// 1 - VFO-B selected
	else
		opr_mode &= VFOA_SELECT;			// 0 - VFO-A selected

	EEPROM_WRITE(TUNE_DIGIT, tune_digit);	// Save tune digit
	EEPROM_WRITE(OPR_MODE, opr_mode);		// Save current operating mode
	lcdclr();
	putst(Settings_Msg);
	lcdcom(SECOND_HALF);
	putst(Saved_Msg);
	tone(tune_tone[3], STD_BEEP);
	delaym(DISPLAY_MSG);
	}

void read_freq(void)
	{
	char i;

	for (i = 0; i < VFO_LEN; i++)
		vfo.vfo_bytes[i] = EEPROM_READ(i);	// Restore VFO-A/B, CW Shift, 160/80/40 Presets

	do	{						// DO/WHILE loop is shorter than a FOR/NEXT loop in this instance.
		bat.batregs[i - BATT_REGS_OFFSET] = EEPROM_READ(i);	// Restore Battery Calibration factor
		i++;
		} while(i < TUNE_DIGIT);

	tune_digit = EEPROM_READ(TUNE_DIGIT);	// Restore tune digit
	opr_mode = EEPROM_READ(OPR_MODE);		// Restore operating mode
	}

long max_min(long num, long max, long min)
	{
// The elegant way of doing this is:
// return (num > max) ? max : (num < min) ? min : num;
// but this took more code.
	if(num > max)
		return max;

	if(num < min)
		return min;

	return num;
	}

void out_of_band(long freq, char band)
	{
	if(opr_mode & ALARM_ENABLE)
		out_of_band_alarm = ((freq < lower_band_limits[band]) || (freq > upper_band_limits[band]));
	}

char band_number(long freq)
	{
	unsigned int band;

	band = (freq >> 8);				// This then avoids doing two long integer comparisons

	if(band >= HIGH_BAND)
		return 2;

	if(band >= MID_BAND)
		return 1;

	return 0;		
	}

void set_band_limits(void)
	{
// Using a local variable and calculating the region once uses 3 more
// bytes than calculating it twice, as here.
	if(opr_mode & USA_REGION)		// 1 = USA Band Selection
		{
		lower_band_limits[0] = USA_160M_LOWER;
		upper_band_limits[1] = USA_80M_UPPER;
		upper_band_limits[2] = USA_40M_UPPER;
		}
	else							// 0 = IARU Region 1 Band selection
		{
		lower_band_limits[0] = IARU_160M_LOWER;
		upper_band_limits[1] = IARU_80M_UPPER;
		upper_band_limits[2] = IARU_40M_UPPER;
		}

	putst(Region_Msg[opr_mode & USA_REGION]);	// Display the current region setting
	}

//*****************
// Main function
//*****************
void main(void)
	{
	int offset;
	signed char i, temp, old_mode = 0;
// Init IO Ports & software modules
//Set up ADC and PORTs
	ANSEL = ANSEL_INIT;				// A/D converter pin select
	PORTA = PORTA_INIT;				// A/D & control signals
	PORTB = PORTB_INIT;				// LCD & SW port
	ADCON1 = ADCON1_INIT;			// PORTA = digital I/O during start
	TRISA = TRISA_INIT;				// Set port direction bits
	TRISB = TRISB_INIT;
	Init_timers();					// Init timers & interrupt system
	Init_ad9833();					// Init DDS
	lcdinit();						// Init LCD
	putst(Juma_TRX);				// Show RX type & version
	lcdcom(SECOND_HALF);
	putst(Version);
	delaym(DISPLAY_MSG);			// Wait for audio ready
	lcdclr();
	putst(OH2NLT);
	lcdcom(SECOND_HALF);
	putst(OH7SV);
	delaym(DISPLAY_MSG);
	read_freq();					// Read EEPROM frequencies and configuration bits
	set_chgen();					// Download soft characters, also clears the display

// System Setup
// =========================================================================================
	if(BUTTON == 0)						// Calibration & Operating mode select
		{
		tone(tune_tone[1], STD_BEEP);	// Make a different sound to the EEPROM save routine
		i = 0;
		putst(System_Msg);
		lcdcom(SECOND_HALF);
		putst(Setup_Msg);				// Display 'System Setup'

		while(BUTTON == 0);				// Hold here until button is released

		do	{
			lcdcom(FIRST_HALF);
			putst(setup_prompt[i]);		// Display menu prompt (Left-hand side of display)
			measure_all();				// Measure parameters.

			switch(i)
				{
				case 0:					// Region Select - IARU/USA
				case 1:					// Beep On/Off
				case 2:					// Out-Of-Band Alarm On/Off
				case 3:					// Auto Voltage Alarm Reset On/Off
					if(pwctr > 0)
						temp = 1;

					if(pwctr < 0)
						temp = 0;

					lcdcom(SECOND_HALF);

					if(i == 0)			// Set Region - IARU Region 1 or USA
						{
						if(pwctr)
							{
							pwctr = 0;	// Clear encoder

							if(temp)
								opr_mode |= USA_REGION;	// USA
							else
								opr_mode &= IARU_REGION;// IARU Region 1
							}

						set_band_limits();				// Adjust band limits, and display region
						}

					if(i == 1)			// Set Beep On/Off
						{
						if(pwctr)
							{
							pwctr = 0;	// Clear encoder

							if(temp)
								opr_mode |= BEEP_ENABLE;
							else
								opr_mode &= BEEP_DISABLE;
							}

						putst(On_Off[((opr_mode & BEEP_ENABLE) == BEEP_ENABLE)]);
						}

					if(i == 2)			// Select Out-Of-Band Alarm On/Off
						{
						if(pwctr)
							{
							pwctr = 0;	// Clear Encoder

							if(temp)
								opr_mode |= ALARM_ENABLE;
							else
								opr_mode &= ALARM_DISABLE;
							}

						putst(On_Off[((opr_mode & ALARM_ENABLE) == ALARM_ENABLE)]);
						}

					if(i == 3)
						{
						if(pwctr)
							{
							pwctr = 0;	// Clear Encoder

							if(temp)
								opr_mode |= AUTO_ENABLE;
							else
								opr_mode &= AUTO_DISABLE;
							}

						putst(On_Off[((opr_mode & AUTO_ENABLE) == AUTO_ENABLE)]);
						}
				break;

				case 4:					// Preset 160m Frequency
				case 5:					// Preset 80m Frequency
				case 6:					// Preset 40m Frequency
					vfo.vfos[i - 2] += (1000L * pwctr);
					pwctr = 0;			// Clear encoder
					vfo.vfos[i - 2] = max_min(vfo.vfos[i - 2], upper_band_limits[i - 4], lower_band_limits[i - 4]);
					display_freq(vfo.vfos[i - 2]);
				break;

				case 7:					// CW Shift Frequency
					CW_SHIFT += (10L * pwctr);
					pwctr = 0;			// Clear encoder
					CW_SHIFT = max_min(CW_SHIFT, MAX_SHIFT, MIN_SHIFT);
					bin2ascii((int)CW_SHIFT, 0);// Use existing integer display option in this function.
				break;

				case 8:					// Voltmeter Calibration
					bin2ascii(batt_voltage, 3);	// Left-justified display: 13.80V
					putst(disp_buffer);	// Display battery voltage
					BATT_FACTOR += pwctr;
					pwctr = 0;			// Clear encoder
					BATT_FACTOR = (int)max_min((long)BATT_FACTOR, MAX_BATT_CAL, MIN_BATT_CAL);
					bin2ascii(BATT_FACTOR, 0);
				break;
				}

			if(i > 3)					// Display buffer messages
				{
				lcdcom(SECOND_HALF);
				putst(disp_buffer);		// Display menu message/result, right-hand side of display.
				}

			if(exit_mode == 1)			// If this was a short button push,
				{
				i++;					// increment the index for the next menu item,

// The elegant way of doing this is: i %= 9; but this uses 30 bytes more!
				if(i > 8)				// and if it is > 8, reset it to 0
					i = 0;

				exit_mode = 0;			// Reset the exit mode selector
				}

			} while(exit_mode < 2);

		tune_digit &= 0x03;				// Keep in range - Fixes battery cal factor corruption bug.
		operating_mode = exit_mode = 0;	// Using in-line code rather than a function saves 5 bytes.
		save_freq();					// Save the current settings
		}
// =========================================================================================

	lcdclr();
	putst(setup_prompt[0]);			// Display: "Region:"
	lcdcom(SECOND_HALF);
	set_band_limits();				// Set band limits and display either "IARU" or "USA"
	delaym(DISPLAY_MSG);			// "Optimised" message display time
	vfo_select = ((opr_mode & VFOB_SELECT) == VFOB_SELECT);
	rit = 0;						// Always start with zero RIT
	old_tune_digit = tune_digit;
	dds_freq = vfo.vfos[vfo_select];// Initialise the synthesiser

// =========================================================================================
/*
 Free running Main Control Loop
 The following loop cycle times were measured with a Tektronix DSO:
 Mode 0 - RIT Off - 9.76mS, RIT On - 7.36mS
 Mode 1 - 9.24mS
 Mode 2 - 7.92mS
 Mode 3 - 9.24mS

 The main loop operates in one of several modes.
 Mode 0	-	Standard Operating Mode
 In this mode, the transceiver will respond to the VFO knob rotation, VFO knob button, RIT switch,
 PTT switch, CW/DSB switch, and keyer/key inputs. The left-hand side of the display will show
 either the graphic S-Meter in receive or the forward/reverse power in transmit. The right-hand
 side of the display will normally show the transceiver frequency as, for example: 3.567.89
 If the RIT function is enabled, the left-hand side of the display will show the S-Meter,
 the right-hand side will show: RIT:1.23 in receive. In transmit, the RIT is disabled, and
 the right-hand side will display the transmit frequency.

 This mode has two sub-modes - RX and TX
 Transmit
 There is no explicit TX/RX signal available from the hardware. There is a CW-TX signal available from
 Port-B bit 4, but this only serves to indicate that the transceiver is in the CW transmit mode, and thus
 the transmit frequency has to be set lower than the receive frequency by the amount of the CW Offset.
 This offset frequency is user adjustable in the System Setup Menu. The transmit mode is enabled by
 pressing the PTT switch when in the DSB mode, or keying the transceiver in the CW mode. The frequency
 displayed differs between transmit and receive by the shift frequency.

 Receive
 In this mode the DDS frequency is governed by the VFO knob as well as an offset component. The offset
 is determined by the RIT setting. If rit_select is on, then the display will show the current offset
 frequency. This frequency is added to or subtracted from the current VFO frequency.

 The actual transceiver frequency is also governed by the VFO selection. This is selected by the vfo_select
 signal, and when low VFO-A is selected, when high VFO-B.

 Also in this mode, whenever the DDS synthesiser is set, a comparison is made and if the frequency is above
 approximately 4MHz, then the high frequency output filters are selected.
 
 With Version 5.2, if the current frequency is outside a recognised amateur band as determined by the
 setting of the IARU/US flag, then the decimal points will blink.

 Mode 1	-	VFO A/B Select Mode
 This mode is entered by means of a medium press of the VFO button. The left-hand side of the display
 will show: VFO-A or VFO-B, the right-hand side will show the frequency. Use the VFO knob to select
 which VFO to use. A brief press of the VFO button will exit this mode, and the frequency loaded into
 the synthesiser will be that of the last selected VFO.

 Mode 2	-	Voltmeter/Tuning Increment Display
 This mode is entered by means of a short press of the VFO button. The left-hand side of the display
 will show the battery voltage: 13.80V, the right-hand side will display the current tuning increment:
 10Hz, 100Hz, 1kHz or 5kHz. The tuning increment can be selected with the VFO tuning knob. The transceiver
 will only respond to the VFO knob for increment adjustment, and the VFO button to exit this mode and
 return the transceiver to Mode 0. Note that with the addition of the under and over voltage alarms in
 Version 5, this mode is forced. The left hand side of the display will show the current input voltage
 and will blink, the right-hand side of the display will be the last displayed data.

 Mode 3	-	Rapid Band-Switch & EEPROM Save Mode
 This mode is entered by means of a long push of the VFO button. The left-hand side of the display will show:
 160m or 80m or 40m, and the right-hand side will display the preset frequency for this band. The band immediately
 displayed is the one nearest the current frequency. Use the VFO knob to select the desired band. A brief press
 of the VFO button will enter the selected frequency into the current VFO, and return you to Mode 0. If you use
 a very long push of the VFO button to exit, then the complete transceiver configuration is saved to the
 EEPROM. This is an easy way to save the current VFO-A/VFO-B frequency settings. Only the VFO knob and VFO button
 controls are recognised.
*/
	for (;;)				// **************  MAIN LOOP  **************
		{
		do	{
			measure_all();							// Measure & store A/D channels

			if((batt_voltage < low_batt) || (batt_voltage > high_batt))
				{
				vcc_alarm = 1;						// Assert alarm state
				operating_mode = 2;					// Force Mode 2 when in alarm state
				low_batt = LOW_BATT + 2;			// Add 20mV of hysteresis
				high_batt = HIGH_BATT - 2;			// Subtract 20mV of hysteresis
				}
			else
				{
				if(vcc_alarm && (opr_mode & AUTO_ENABLE))
					{								// Auto_reset mode
					operating_mode = 0;				// Reset to Mode 0
					vcc_alarm = 0;					// Reset alarm state
					}

				low_batt = LOW_BATT;				// Restore original thresholds
				high_batt = HIGH_BATT;
				}

			if((old_mode != operating_mode) && !rit_select)
				{
				old_mode = operating_mode;
				tone(tune_tone[operating_mode], STD_BEEP);
				}

			aux_out = (vcc_alarm || out_of_band_alarm);
			tune_ad9833(max_min(dds_freq, TUNE_HIGH_LIMIT, TUNE_LOW_LIMIT), 1);
			lcdcom(FIRST_HALF);

			switch(operating_mode)					// Mode obtained from 1mS timer interrupt (dcrxtim.c)
				{
				case 0:								// Normal Operating Mode
// Display meters and frequencies
					draw_s_meter(smeter);

					if((portb_copy & TX) == TX)		// DSB Transmit/Receive & CW Receive
						{
						if(RIT_ON)
							rit_select = 0;
						else
							rit_select = 1;

						if(rit_select)				// RIT is On
							{
							rit += (TUNE_STEP_LOW * pwctr);		// Add +/- low steps (Modified)
							pwctr = 0;
							rit = max_min(rit, RIT_HIGH, RIT_LOW);	// This consumes 9 words less than the in-line version
							offset = rit;
							bin2ascii(rit, 5);		// Special RIT display
							}
						else						// Normal receive
							{
							vfo.vfos[vfo_select] += (tuning_step[tune_digit] * pwctr);
							pwctr = 0;
							vfo.vfos[vfo_select] = max_min(vfo.vfos[vfo_select], TUNE_HIGH_LIMIT, TUNE_LOW_LIMIT); 
							offset = 0;
							}
						}
					else							// CW Transmit
						offset = (int)-CW_SHIFT;

					dds_freq = vfo.vfos[vfo_select] + offset;
					out_of_band(dds_freq, band_number(dds_freq));
					lcdcom(SECOND_HALF);			// Select right-hand side of display

					if(!rit_select)					// RIT is Off
						display_freq(dds_freq);		// Normal frequency display

					putst(disp_buffer);				// Display frequency or RIT shift
				break;

				case 1:			// VFO A/B Select
					if(rit_select)					// Reset RIT to zero
						{
						rit = 0;
						pwctr = 0;
						exit_mode = SHORT_PUSH;
						break;
						}

					if(pwctr)
						vfo_select = (pwctr > 0);

					pwctr = 0;						// Reset encoder
					putst(VFO_Msg[vfo_select]);
					display_freq(vfo.vfos[vfo_select]);
					lcdcom(SECOND_HALF);			// Select right-hand half of display,
					putst(disp_buffer);				// and display current frequency.
				break;

				case 2:			// Voltmeter/Tuning Increment Select - 10/100/1,000,5,000Hz
					if(rit_select)
						pwctr = 0;
					else
						{
						tune_digit += pwctr;		// Get new tune increment
						pwctr = 0;

						if(tune_digit > 3)			// Keep within range of 0 - 3,
							tune_digit = 3;

						if(tune_digit < 0)
							tune_digit = 0;

						if(old_tune_digit != tune_digit)
							{
							tone(tune_tone[tune_digit], STD_BEEP);
							old_tune_digit = tune_digit;
							}
						}

					bin2ascii(batt_voltage, 3);		// Left-justified display: 13.80V
					putst(disp_buffer);				// Display battery voltage
					lcdcom(SECOND_HALF);			// Select right-hand half of display

					if(vcc_alarm)
						{
						if((blink_timer > BLINK_TIME))
							putst(Blank_Line);
						else
							putst(Hi_Lo_V);
						}
					else
						{
						putst(Step_Msg);
						putst(tuning_increment[tune_digit]);
						}
				break;

				case 3:			// Rapid Band-Switch/EEPROM Save
					if(rit_select)					// Exit if RIT is on.
						{
						pwctr = 0;
						exit_mode = SHORT_PUSH;
						break;
						}

					i = 2 + band_number(vfo.vfos[vfo_select]);
					i += pwctr;						// Update the selection.
					pwctr = 0;

					if(i > 4)						// If selection is > 40m,
						i = 4;						// make it 40m.

					if(i < 2)						// If selection is < 160m
						i = 2;						// make it 160m.

					putst(setup_prompt[i + 2]);		// Display currently selected band,
					display_freq(vfo.vfos[i]);		// and get its preset frequency.
					lcdcom(SECOND_HALF);			// Select right-hand half of display,
					putst(disp_buffer);				// and display currently selected frequency.
					vfo.vfos[vfo_select] = vfo.vfos[i];
				break;
				}

			if(exit_mode == MEDIUM_PUSH && operating_mode == 1)	// VFO Copy Mode
				{
				exit_mode = 0;						// Reset exit_mode to return to VFO A/B display

				if(vfo_select)						// VFO-B is the active VFO
					VFO_B = VFO_A;					// So copy VFO-A to VFO-B
				else
					VFO_A = VFO_B;					// Else copy VFO-B to VFO-A
				}
				
			} while(exit_mode == 0);				// Repeat until exit.

		if((exit_mode == LONG_PUSH) && !rit_select)	// If button push was a long push, save the new settings
			save_freq();

		if((opr_mode & AUTO_ENABLE) == 0)
			vcc_alarm = 0;							// Reset hi-lo voltage alarm if in manual reset mode

		operating_mode = exit_mode = 0;				// Using in-line code rather than a function saves 5 bytes.
		}
	}

