/*
 * DC Receiver controller board definitions
 * 
 * OH2NLT / Juha Niinikoski 25.13.2003
 *
 * Tone I/O added 03.02.2005
 * TX signal added 20.06.2005
 * Band Select 03.07.2005
 * 16F88 alternative added, VFO select added 06.10.2005
 */
#include <htc.h>

// Macro Definitions
#define display_freq(freq)	tune_ad9833(freq, 0)

// General Definitions & Constants
#define						F88				// Some modules use this to generate 16F88 code
#define NUM_VFO				6				// Number of VFOs
#define VFO_LEN				24				// 4 bytes/long integer
#define VFO_A				(vfo.vfos[0])	// Define VFO structures
#define VFO_B				(vfo.vfos[1])
#define _160M				(vfo.vfos[2])	// 160m Preset Frequency
#define _80M				(vfo.vfos[3])	// 80m Preset Frequency
#define _40M				(vfo.vfos[4])	// 40m Preset Frequency
#define CW_SHIFT			(vfo.vfos[5])	// CW Shift frequency
#define BATT_CAL			1024			// Default battery voltage calibration factor
#define BATT_FACTOR			(bat.battery[0])
#define BATT_VOLTAGE		(bat.battery[1])

#define BATT_REGS_OFFSET	24			// Offset for indexing bat.batregs[] array when saving/retoring EEPROM data
#define BATT_ADDR			24			// Battery Calibration address
#define TUNE_DIGIT			26			// Tune digit address
#define OPR_MODE			27			// Operating mode bits address

// Tuning Limits
#define TUNE_LOW_LIMIT		0L			// Test setup can go down to 0 Hz
#define TUNE_HIGH_LIMIT 	8000000L	// 8 MHz upper limit

// Battery Calibration Limits
#define MAX_BATT_CAL		1130		// For nominal calibration = 12.50V with 13.80V input.
#define MIN_BATT_CAL		930			// For nominal calibration = 15.19V with 13.80V input.
#define LOW_BATT			1070		// Low battery alarm - 10.70V
#define HIGH_BATT			1530		// High battery alarm - 15.30V

// Tuning Steps
#define DEF_STEP 			0			// Default / start up tune step, 10Hz
#define TUNE_STEP_LOW 		10			// 10 Hz
#define TX_SHIFT			700L		// 700 Hz, when CW TX subtract this
#define MIN_SHIFT			300L
#define MAX_SHIFT			1000L
#define RIT_HIGH			9990		// RIT High limit
#define RIT_LOW				-9990		// RIT Low limit

// User Interface Timers
#define DISPLAY_MSG			1500		// Message display time, 1500mS
#define STD_BEEP			50			// Standard beep length, 50mS
#define BLINK_PERIOD		1200		// Blink period
#define BLINK_TIME			BLINK_PERIOD / 2
#define AVG_SAMPLES			32
#define MULTIPLIER			(2048L / AVG_SAMPLES)
#define A_D_INHIBIT_DELAY	80			// 50 is very slightly too short an inhibit time, 80 is about right

// Constants
#define USA_REGION			0b00000001	// IARU/USA tuning select 0 = IARU, 1 = USA
#define IARU_REGION			0b00011110
#define BEEP_ENABLE			0b00000010	// Beep enable 1 = Beeps On, 0 = Beeps Off
#define BEEP_DISABLE		0b00011101
#define ALARM_ENABLE		0b00000100	// Out-Of-Band Alarm Mask 1 = Enable 0 = Disable
#define ALARM_DISABLE		0b00011011
#define AUTO_ENABLE			0b00001000	// Auto Reset On/Off
#define AUTO_DISABLE		0b00010111
#define VFOB_SELECT			0b00010000	// VFO-B Select Mask
#define VFOA_SELECT 		0b00001111

// Tone Definitions
#if 0
#define LOW_TONE			2500		// 200 Hz
#define MID_TONE			833			// 600 Hz
#define HIGH_TONE			500			// 1 kHz
#endif

#define LOW_TONE			1911		// C 261,6 Hz
#define MID_TONE			1517		// E 329,6 Hz
#define HIGH_TONE			1276		// G 392 Hz
#define HIGH2_TONE			1012		// B 493,8 Hz

// Standard Frequencies
#define IARU_160M_LOWER		1810000L
#define IARU_80M_LOWER		3500000L
#define IARU_40M_LOWER		7000000L
#define IARU_160M_UPPER		2000000L
#define IARU_80M_UPPER		3800000L
#define IARU_40M_UPPER		7200000L
#define USA_160M_LOWER		1800000L
#define USA_80M_UPPER		4000000L
#define USA_40M_UPPER		7300000L

// Rapid Band-Switch Frequencies
#define _160M_CENTRE		1900000L
#define _80M_CENTRE			3600000L
#define _40M_CENTRE			7100000L
#define HIGH_BAND			(5500000L >> 8)
#define MID_BAND			(2750000L >> 8)

/*
 The band change point is actually calculated by comparing the upper 14 bits of the DDS word
 with this constant. The actual point at which the band change occurs, assuming a reference
 frequency of exactly 20MHz, is: (0xCD0 << 14) * 20000000 / 2^28 or: (3280 * 2^14) * 20000000 / 2^28
 which is: 4,003,906.25Hz For a constant of 0xCD1, the frequency would be: 4,005,126.953Hz
 For a constant of 0xCCF, the frequency would be: 4,002,685.547Hz. The increments are: 1,220.703Hz
 The crystal oscillator has a nominal tolerance of +/-30ppm. Thus, for the lowest frequency crystal
 of 20,000,000 - 600 the nominal constant results in a changeover frequency of: 4,003,786.13Hz
 For the highest frequency crystal, the changeover frequency is: 4,004,026.37Hz. There is thus an
 'uncertainty' of +/- 120Hz, which is insignificant. By using an integer comparison rather than a
 long, 9 program words are saved.
*/
#define FILTER_SEL			0xCD0		// Band change frequency, 4.004MHz

//	Port A used for A/D inputs & control signals
//#define	ADCON1_INIT 	0x07		// Default state, PORTA = digital I/O
//#define ADCON1_MEAS		0x84		// Right justified, 3/1 mode
#ifdef F88
#define ANSEL_INIT			0x00		// All pins for digital I/O
#define	ANSEL_MEAS			0x03		// AN0 and AN1 in use
#define	ADCON1_INIT 		0x80		// Default state, internal Vref

#else
#define	ADCON1_INIT 		0x87		// Default state, PORTA = digital I/O
#define ADCON1_MEAS			0x84		// Right justified, 3/1 mode
#endif

#define ADCON0_CH0			0xC1		// Setup for CH0
#define ADCON0_CH1			0xC9		// Setup for CC1
#define ADCON0_OFF			0x00		// A/D system Off

#define	TRISA_INIT			0b11100011
#define	PORTA_INIT			0x00

#define TONE_DIR			TRISA0		// Tone I/O direction bit
#define TONE_OUT			RA0			// Send beep tone to battery voltage measurement circuit
#define	LCD_RS				RA3
#define	LCD_E				RA4			// LCD control signals

#define AD_FSYNC			RA2			// DDS Chip signals
#define AD_SCLK				RB1
#define AD_SDATA			RB0

#define RIT_ON				RA5			// RIT Switch Sense, active low. 0 = RIT On, 1 = RIT Off

//	Port B used for LCD Data & Switch inputs & DDS control
#define PORTB_INIT			0b00000000	// PORTB init data
#define TRISB_INIT			0x00		// Default state, all outputs
#define TRISB_READ			0b11110000	// Switch read state, do not change DDS control bits!

#define	LCD_DATA			PORTB

// PORTB read bit masks
#define BUTTON_1			0b00100000	// Encoder push button
#define BUTTON				(portb_copy & BUTTON_1)
#define TX					0b00010000	// TX signal, 0 = CW TX

#define AUX_OUT				RB2			// AUX output flip-flop drive
#define BAND				RB3			// Band select flip-flop drive

// VFO Push Button Times
#define SHORT_PRESS			50			// 50mS
#define MEDIUM_PRESS		500			// 500mS
#define LONG_PRESS			1500		// 1,500mS
#define SHORT_PUSH			1
#define MEDIUM_PUSH			2
#define LONG_PUSH			3

// Define unsigned data types
#ifndef _BYTE_DEFINED_
#define _BYTE_DEFINED_
typedef unsigned char BYTE;
typedef unsigned int WORD;
typedef unsigned long LWORD;
#endif

// Define Boolean data type
#ifndef _BOOL_DEFINED_
#define _BOOL_DEFINED_
typedef BYTE BOOL;
#define FALSE				0
#define TRUE 				1
#endif

// Define null pointer
#ifndef NULL
#define NULL				(void *)0
#endif

